package com.usky.http.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.nacos.common.utils.JacksonUtils;
import com.alibaba.nacos.shaded.com.google.gson.JsonObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.ctg.ag.sdk.biz.AepDeviceCommandLwmProfileClient;
import com.ctg.ag.sdk.biz.aep_device_command_lwm_profile.CreateCommandLwm2mProfileRequest;
import com.ctg.ag.sdk.biz.aep_device_command_lwm_profile.CreateCommandLwm2mProfileResponse;
import com.usky.common.core.exception.BusinessException;
import com.usky.common.core.util.JsonUtils;
import com.usky.http.domain.DmpDevice;
import com.usky.http.domain.TcAepCommand;
import com.usky.http.mapper.TcAepCommandMapper;
import com.usky.http.service.DmpDeviceService;
import com.usky.http.service.TcAepCommandService;
import com.usky.common.mybatis.core.AbstractCrudService;
import com.usky.http.service.config.mqtt.MqttOutConfig;
import com.usky.http.service.vo.TCAepDeviceCommendRequestVO;
import com.usky.http.service.vo.TcAepJTPDeviceInfoAlarmRequestVO;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * <p>
 * 电信对接表 服务实现类
 * </p>
 *
 * @author ya
 * @since 2024-06-04
 */
@Service
@Slf4j
public class TcAepCommandServiceImpl extends AbstractCrudService<TcAepCommandMapper, TcAepCommand> implements TcAepCommandService {
    @Autowired
    private DmpDeviceService dmpDeviceService;

    @Resource
    private MqttOutConfig.MqttGateway mqttGateway;

    @Override
    public void tcAepDeviceCommand(TCAepDeviceCommendRequestVO requestVO) throws Exception {
        AepDeviceCommandLwmProfileClient client = AepDeviceCommandLwmProfileClient.newClient().appKey("izWiESDywH2").appSecret("vlOdjIbDet").build();

        CreateCommandLwm2mProfileRequest request = new CreateCommandLwm2mProfileRequest();
        request.setParamMasterKey("a97c2ece31084f19b2b95a859610e6d5");
        Map<String, Object> map = new HashMap<>();
        String deviceId = requestVO.getDeviceId();
        map.put("deviceId", requestVO.getTcDeviceId());
        map.put("operator", "admin");
        map.put("productId", requestVO.getProductId());
        map.put("ttl",0);
        Map<String, Object> payload = new HashMap<>();
        HashMap<Object, Object> objectObjectHashMap = new HashMap<>();
        objectObjectHashMap.put("deviceid", deviceId.substring(3));
        objectObjectHashMap.put("devicetype",requestVO.getDeviceType());
        objectObjectHashMap.put("soundtype",4);
        objectObjectHashMap.put("sirentime",30);
        payload.put("serviceId", "Control");
        payload.put("method", "SIREN_ALARM");
        payload.put("paras", objectObjectHashMap);
        map.put("command", payload);
        request.setBody(JacksonUtils.toJsonBytes(map));
        CreateCommandLwm2mProfileResponse createCommandResponse = client.CreateCommandLwm2mProfile(request);
        String logInfo = "";
        if (createCommandResponse.getStatusCode() != 200) {
            logInfo = String.format(deviceId,": ",createCommandResponse.getStatusCode(),createCommandResponse.getMessage());
        }else{
            logInfo = String.format(deviceId,": ","下发命令成功");
        }
        log.info(logInfo);
    }

    @Override
    public void tcAepJTYDeviceInfo(String requestBody){
        System.out.println("----requestBody: "+requestBody);

        Map map = JsonUtils.fromJson(requestBody,Map.class);
        String deviceId = map.get("IMEI").toString();
        LambdaQueryWrapper<DmpDevice> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpDevice::getDeviceId,deviceId);
        DmpDevice one = dmpDeviceService.getOne(queryWrapper);
        String companyCode = one.getCompanyCode();
        String deviceName = one.getDeviceName();
        String deviceType = one.getDeviceType().toString();
        String infoTopic = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/tcInfo";
        String infoTopic3 = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/info";
        String alarmTopic3 = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/alarm";
        String alarmTopic = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/tcAlarm";
        //String timestamp = String.valueOf(LocalDateTime.now().toEpochSecond(ZoneOffset.of("+8")));
        String tp = map.get("timestamp").toString();
        String timestamp = tp.substring(0,10);

        String messageType = map.get("messageType").toString();

        if(messageType.equals("dataReport")){
            Map<String,Object> dataMap = JSONObject.parseObject(JSONObject.toJSONString(map.get("payload")));
            if(Objects.nonNull(dataMap.get("smoke_state"))){ //alarm
                String smokeState = dataMap.get("smoke_state").toString();
                if(smokeState.equals("0")){
                    String deviceStatus = "0";
                    String battery = dataMap.get("battery_value").toString();
                    String signal = dataMap.get("signalPower").toString();
                    String temperature = "0.0";
                    String smoke = dataMap.get("smoke_value").toString();
                    String deviceAlarmData = "{\"devId\":\""+deviceId+"\",\"connType\":\"N\""+",\"deviceName\":\""+deviceName+"\",\"alarams\":[{\"dp\":[{\"property\":\"烟雾报警\",\"serial\":\"311\",\"value\":\"报警\",\"status\":\"0\" }],\"deviceName\":\""+
                            deviceName+"\",\"timeStamp\":\""+timestamp+"\",\"deviceModel\":\"\",\"deviceType\":\""+deviceType+"\",\"devId\":\""+deviceId+"\"}],\"type\":\"ALARM\",\"timeStamp\":\""+timestamp+"\"}";

                    mqttGateway.sendToMqtt(alarmTopic,deviceAlarmData);
                    mqttGateway.sendToMqtt(alarmTopic3,deviceAlarmData);
                }

            }else{ //info
                String deviceStatus = "0";
                String battery = dataMap.get("battery_value").toString();
                String signal = dataMap.get("rsrp").toString();
                String temperature = "0.0";
                String smoke = "0";
                String deviceInfoData = "{\"devId\":\""+deviceId+"\",\"connType\":\"N\""+",\"deviceName\":\""+deviceName+"\",\"devs\":[{\"dp\":[{\"property\":\"状态\",\"serial\":\"301\",\"value\":\""
                        +deviceStatus+"\"},{\"property\":\"剩余电量\",\"serial\":\"302\",\"value\":\""+battery+"\"},{\"property\":\"无线信号\",\"serial\":\"303\",\"value\":\""+signal+
                        "\"},{\"property\":\"温度\",\"serial\":\"304\",\"value\":\""+temperature+"\"},{\"property\":\"浓度\",\"serial\":\"305\",\"value\":\""+smoke+"\"}],\"deviceName\":\""+deviceName+"\",\"timeStamp\":\""+timestamp+"\",\"deviceModel\":\"\",\"deviceType\":\""+deviceType+"\",\"devId\":\""+deviceId+"\"}],\"type\":\"INFO\",\"timeStamp\":\""+timestamp+"\"}";

                mqttGateway.sendToMqtt(infoTopic,deviceInfoData);
                mqttGateway.sendToMqtt(infoTopic3,deviceInfoData);

            }

        }else if(messageType.equals("eventReport")){ //alarm

        }


    }

    @Override
    public void tcAepTbsnDeviceInfoAlarm(String requestBody){
        Map map = JsonUtils.fromJson(requestBody,Map.class);
        String deviceId = map.get("IMEI").toString();
        LambdaQueryWrapper<DmpDevice> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.eq(DmpDevice::getDeviceId,deviceId);
        DmpDevice one = dmpDeviceService.getOne(queryWrapper);
        String companyCode = one.getCompanyCode();
        String deviceName = one.getDeviceName();
        String deviceType = one.getDeviceType().toString();
        String infoTopic = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/tcInfo";
        String infoTopic3 = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/info";
        String alarmTopic3 = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/alarm";
        String alarmTopic = "/usky/ytDP0003/"+companyCode+"/"+deviceId+"/tcAlarm";
        String timestamp = map.get("timestamp").toString().substring(0,10);

        String messageType = map.get("messageType").toString();

        if(messageType.equals("dataReport")){ //info
            Map<String,Object> dataMap = JSONObject.parseObject(JSONObject.toJSONString(map.get("payload")));
            String deviceStatus = "0";
            String battery = "0.0";
            String signal = dataMap.get("rsrp").toString();
            String temperature = "0.0";
            String smoke = "0";
            String deviceInfoData = "{\"devId\":\""+deviceId+"\",\"connType\":\"N\""+",\"deviceName\":\""+deviceName+"\",\"devs\":[{\"dp\":[{\"property\":\"状态\",\"serial\":\"301\",\"value\":\""
                    +deviceStatus+"\"},{\"property\":\"剩余电量\",\"serial\":\"302\",\"value\":\""+battery+"\"},{\"property\":\"无线信号\",\"serial\":\"303\",\"value\":\""+signal+
                    "\"},{\"property\":\"温度\",\"serial\":\"304\",\"value\":\""+temperature+"\"},{\"property\":\"浓度\",\"serial\":\"305\",\"value\":\""+smoke+"\"}],\"deviceName\":\""+deviceName+"\",\"timeStamp\":\""+timestamp+"\",\"deviceModel\":\"\",\"deviceType\":\""+deviceType+"\",\"devId\":\""+deviceId+"\"}],\"type\":\"INFO\",\"timeStamp\":\""+timestamp+"\"}";

            mqttGateway.sendToMqtt(infoTopic,deviceInfoData);
            mqttGateway.sendToMqtt(infoTopic3,deviceInfoData);

        }else if(messageType.equals("eventReport")){ //alarm
            Map<String,Object> dataMap = JSONObject.parseObject(JSONObject.toJSONString(map.get("eventContent")));
            String alarmState = dataMap.get("alarm").toString();
            if(alarmState.equals("1")){
                String deviceAlarmData = "{\"devId\":\""+deviceId+"\",\"connType\":\"N\""+",\"deviceName\":\""+deviceName+"\",\"alarams\":[{\"dp\":[{\"property\":\"手报\",\"serial\":\"311\",\"value\":\"报警\",\"status\":\"0\" }],\"deviceName\":\""+
                        deviceName+"\",\"timeStamp\":\""+timestamp+"\",\"deviceModel\":\"\",\"deviceType\":\""+deviceType+"\",\"devId\":\""+deviceId+"\"}],\"type\":\"ALARM\",\"timeStamp\":\""+timestamp+"\"}";

                mqttGateway.sendToMqtt(alarmTopic,deviceAlarmData);
                mqttGateway.sendToMqtt(alarmTopic3,deviceAlarmData);
            }
        }

    }


}
